-- fourbit_counter.vhd
--
-- This is a simple 4-bit (Ripple) binary counter made up
-- of four T flip-flops. It also includes a clock divider
-- to bring down the input clock signal from 50MHz to about 1Hz.
--
-- NOTE: We cannot use ALTPLL here because the frequency
-- that we need is too low; that is why we generate a low
-- frequency clock with counters;
--
-- NOTE: To get rid of that "Synopsys Design Constraints File file not found"
-- you can just Create a Default TimeQuest SDC File as described in the 
-- My_First_Fpga.pdf from the CD from Terasic that comes with the DE1-SoC board.
-- The steps to do that are as follows:
-- 1. Open the TimeQuest Timing Analyzer by choosing Tools -> TimeQuest Timing Analyzer
-- 2. Choose File -> New SDC file. The editor opens.
-- 3. Type the following code into the file (file is open within the Quartus window):
--    create_clock -name "clk_50" -period 20.000ns [get_ports {clk_50}]
--    derive_pll_clocks
--    derive_clock_uncertainty
-- Save the SDC as fourbit_counder.sdc (same name as the top-level entity)
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity fourbit_counter is
  port ( 
    clk_50 : in STD_LOGIC;
    Q : out STD_LOGIC_VECTOR (3 downto 0)
  );
end fourbit_counter;


architecture my_structural of fourbit_counter is


  component my_tff
    port ( 
      T : in STD_LOGIC;
      clk : in STD_LOGIC;
      Q, QN : out STD_LOGIC
    );
  end component;

  component clk_divider
    port ( 
      CLK_IN : in STD_LOGIC;
      CLK_OUT : out STD_LOGIC
    );
  end component;

  signal all_T, S0, S1, S2, S3 : STD_LOGIC;
  signal clk_1Hz : STD_LOGIC;

begin

-- use signal all_T to drive input T of all T flip-flops to logic '1';
all_T <= '1';
 
CLOCK: clk_divider port map (CLK_IN => clk_50, CLK_OUT => clk_1Hz);

TFF0: my_tff port map (T => all_T, clk => clk_1Hz, Q => Q(0), QN => S0);
TFF1: my_tff port map (T => all_T, clk => S0, Q => Q(1), QN => S1);
TFF2: my_tff port map (T => all_T, clk => S1, Q => Q(2), QN => S2);
TFF3: my_tff port map (T => all_T, clk => S2, Q => Q(3), QN => S3);

end my_structural;

